@@ -66,11 +66,20 @@ BatchExtract::~BatchExtract()
 void BatchExtract::addExtraction(Kerfuffle::Archive* archive)
 {
     QString destination = destinationFolder();
+    const bool isSingleFolderRPM = (archive->isSingleFolderArchive() &&
+                                   (archive->mimeType().name() == QLatin1String("application/x-rpm")));
 
-    if ((autoSubfolder()) && (!archive->isSingleFolderArchive())) {
+    if ((autoSubfolder()) && (!archive->isSingleFolderArchive() || isSingleFolderRPM)) {
         const QDir d(destination);
         QString subfolderName = archive->subfolderName();
 
+        // Special case for single folder RPM archives.
+        // We don't want the autodetected folder to have a meaningless "usr" name.
+        if (isSingleFolderRPM && subfolderName == QStringLiteral("usr")) {
+            qCDebug(ARK) << "Detected single folder RPM archive. Using archive basename as subfolder name";
+            subfolderName = QFileInfo(archive->fileName()).completeBaseName();
+        }
+
         if (d.exists(subfolderName)) {
             subfolderName = KIO::suggestName(QUrl::fromUserInput(destination, QString(), QUrl::AssumeLocalFile), subfolderName);
         }
@@ -85,7 +94,7 @@ void BatchExtract::addExtraction(Kerfuffle::Archive* archive)
 
     Kerfuffle::ExtractJob *job = archive->copyFiles(QVariantList(), destination, options);
 
-    qCDebug(ARK) << QString(QStringLiteral("Registering job from archive %1, to %2, preservePaths %3")).arg(archive->fileName()).arg(destination).arg(preservePaths());
+    qCDebug(ARK) << QString(QStringLiteral("Registering job from archive %1, to %2, preservePaths %3")).arg(archive->fileName(), destination, QString::number(preservePaths()));
 
     addSubjob(job);
 
@@ -209,7 +218,7 @@ bool BatchExtract::addInput(const QUrl& url)
     Kerfuffle::Archive *archive = Kerfuffle::Archive::create(url.toDisplayString(QUrl::PreferLocalFile), this);
     Q_ASSERT(archive);
 
-    if (!QFileInfo(url.toDisplayString(QUrl::PreferLocalFile)).exists()) {
+    if (!QFileInfo::exists(url.toDisplayString(QUrl::PreferLocalFile))) {
         m_failedFiles.append(url.fileName());
         return false;
     }
