@@ -27,6 +27,7 @@
 #include "kerfuffle/archive_kerfuffle.h"
 #include "kerfuffle/createdialog.h"
 #include "kerfuffle/settingspage.h"
+#include "mimetypes.h"
 #include "part/interface.h"
 
 #include <KPluginFactory>
@@ -51,6 +52,7 @@
 #include <QMimeData>
 #include <QPointer>
 #include <QRegularExpression>
+#include <QStatusBar>
 
 static bool isValidArchiveDrag(const QMimeData *data)
 {
@@ -63,7 +65,6 @@ MainWindow::MainWindow(QWidget *)
     setXMLFile(QStringLiteral("arkui.rc"));
 
     setupActions();
-    statusBar();
 
     resize(640, 480);
 
@@ -152,6 +153,7 @@ bool MainWindow::loadPart()
     m_part->setObjectName(QStringLiteral("ArkPart"));
     setCentralWidget(m_part->widget());
     createGUI(m_part);
+    statusBar()->hide();
 
     connect(m_part, SIGNAL(busy()), this, SLOT(updateActions()));
     connect(m_part, SIGNAL(ready()), this, SLOT(updateActions()));
@@ -162,22 +164,21 @@ bool MainWindow::loadPart()
 
 void MainWindow::setupActions()
 {
-    m_newAction = KStandardAction::openNew(this, SLOT(newArchive()), actionCollection());
-    m_openAction = KStandardAction::open(this, SLOT(openArchive()), actionCollection());
-    KStandardAction::quit(this, SLOT(quit()), actionCollection());
+    m_newAction = actionCollection()->addAction(KStandardAction::New, QStringLiteral("ark_file_new"), this, SLOT(newArchive()));
+    m_openAction = actionCollection()->addAction(KStandardAction::Open, QStringLiteral("ark_file_open"), this, SLOT(openArchive()));
+    actionCollection()->addAction(KStandardAction::Quit, QStringLiteral("ark_quit"), this, SLOT(quit()));
+
+    m_recentFilesAction = KStandardAction::openRecent(this, SLOT(openUrl(QUrl)), Q_NULLPTR);
+    actionCollection()->addAction(QStringLiteral("ark_file_open_recent"), m_recentFilesAction);
 
-    m_recentFilesAction = KStandardAction::openRecent(this, SLOT(openUrl(QUrl)), actionCollection());
     m_recentFilesAction->setToolBarMode(KRecentFilesAction::MenuMode);
     m_recentFilesAction->setToolButtonPopupMode(QToolButton::DelayedPopup);
     m_recentFilesAction->setIconText(i18nc("action, to open an archive", "Open"));
-    m_recentFilesAction->setStatusTip(i18n("Click to open an archive, click and hold to open a recently-opened archive"));
     m_recentFilesAction->setToolTip(i18n("Open an archive"));
     m_recentFilesAction->loadEntries(KSharedConfig::openConfig()->group("Recent Files"));
     connect(m_recentFilesAction, SIGNAL(triggered()),
             this, SLOT(openArchive()));
 
-    createStandardStatusBarAction();
-
     KStandardAction::configureToolbars(this, SLOT(editToolbars()), actionCollection());
     KStandardAction::keyBindings(this, SLOT(editKeyBindings()), actionCollection());
     KStandardAction::preferences(this, SLOT(showSettings()), actionCollection());
@@ -222,25 +223,8 @@ void MainWindow::openArchive()
     Q_UNUSED(iface);
 
     QPointer<QFileDialog> dlg = new QFileDialog(this, i18nc("to open an archive", "Open Archive"));
-    dlg->setMimeTypeFilters(Kerfuffle::supportedMimeTypes().toList());
 
-    QStringList filters = dlg->nameFilters();
-    filters.removeDuplicates();
-    filters.sort(Qt::CaseInsensitive);
-
-    // Create the "All supported archives" filter
-    QRegularExpression rx(QStringLiteral("(\\*\\.[a-z]+\\.*[a-z0-9]*)+"));
-    QString allArchives(i18n("All supported archives ("));
-    foreach(const QString &s, filters)
-    {
-        QRegularExpressionMatchIterator i = rx.globalMatch(s);
-        while (i.hasNext()) {
-            QRegularExpressionMatch match = i.next();
-            allArchives.append(match.captured(1) + QLatin1Char(' '));
-        }
-    }
-    filters.prepend(allArchives + QLatin1Char(')'));
-    dlg->setNameFilters(filters);
+    dlg->setMimeTypeFilters(Kerfuffle::supportedMimeTypes());
 
     dlg->setFileMode(QFileDialog::ExistingFile);
     dlg->setAcceptMode(QFileDialog::AcceptOpen);
@@ -322,12 +306,12 @@ void MainWindow::newArchive()
         const QString password = dialog.data()->password();
 
         qCDebug(ARK) << "CreateDialog returned URL:" << saveFileUrl.toString();
-        qCDebug(ARK) << "CreateDialog returned mime:" << dialog.data()->currentMimeFilter();
+        qCDebug(ARK) << "CreateDialog returned mime:" << dialog.data()->currentFilterMimeType().name();
 
         m_openArgs.metaData()[QStringLiteral("createNewArchive")] = QStringLiteral("true");
         m_openArgs.metaData()[QStringLiteral("encryptionPassword")] = password;
 
-        if (dialog.data()->isHeaderEncryptionChecked()) {
+        if (dialog.data()->isHeaderEncryptionEnabled()) {
             m_openArgs.metaData()[QStringLiteral("encryptHeader")] = QStringLiteral("true");
         }
 
