@@ -43,6 +43,7 @@
 #include <QDateTime>
 #include <QDebug>
 #include <QDir>
+#include <QDirIterator>
 #include <QEventLoop>
 #include <QFile>
 #include <QProcess>
@@ -143,136 +144,32 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
     qCDebug(ARK) << Q_FUNC_INFO << "to" << destinationDirectory;
 
     cacheParameterList();
-
     m_operationMode = Copy;
+    const QStringList extractArgs = m_param.value(ExtractArgs).toStringList();
 
-    //start preparing the argument list
-    QStringList args = m_param.value(ExtractArgs).toStringList();
-
-    //now replace the various elements in the list
-    for (int i = 0; i < args.size(); ++i) {
-        QString argument = args.at(i);
-        qCDebug(ARK) << "Processing argument " << argument;
-
-        if (argument == QLatin1String( "$Archive" )) {
-            args[i] = filename();
-        }
-
-        if (argument == QLatin1String( "$PreservePathSwitch" )) {
-            QStringList replacementFlags = m_param.value(PreservePathSwitch).toStringList();
-            Q_ASSERT(replacementFlags.size() == 2);
-
-            bool preservePaths = options.value(QStringLiteral( "PreservePaths" )).toBool();
-            QString theReplacement;
-            if (preservePaths) {
-                theReplacement = replacementFlags.at(0);
-            } else {
-                theReplacement = replacementFlags.at(1);
-            }
-
-            if (theReplacement.isEmpty()) {
-                args.removeAt(i);
-                --i; //decrement to compensate for the variable we removed
-            } else {
-                //but in this case we don't have to decrement, we just
-                //replace it
-                args[i] = theReplacement;
-            }
-        }
-
-        if (argument == QLatin1String( "$PasswordSwitch" )) {
-            //if the PasswordSwitch argument has been added, we at least
-            //assume that the format of the switch has been added as well
-            Q_ASSERT(m_param.contains(PasswordSwitch));
-
-            //we will decrement i afterwards
-            args.removeAt(i);
-
-            //if we get a hint about this being a password protected archive, ask about
-            //the password in advance.
-            if ((options.value(QStringLiteral("PasswordProtectedHint")).toBool()) &&
-                (password().isEmpty())) {
-                qCDebug(ARK) << "Password hint enabled, querying user";
-
-                Kerfuffle::PasswordNeededQuery query(filename());
-                emit userQuery(&query);
-                query.waitForResponse();
-
-                if (query.responseCancelled()) {
-                    emit cancelled();
-                    // There is no process running, so finished() must be emitted manually.
-                    emit finished(false);
-                    failOperation();
-                    return false;
-                }
-                setPassword(query.password());
-            }
-
-            QString pass = password();
-
-            if (!pass.isEmpty()) {
-                QStringList theSwitch = m_param.value(PasswordSwitch).toStringList();
-                for (int j = 0; j < theSwitch.size(); ++j) {
-                    //get the argument part
-                    QString newArg = theSwitch.at(j);
-
-                    //substitute the $Path
-                    newArg.replace(QLatin1String( "$Password" ), pass);
-
-                    //put it in the arg list
-                    args.insert(i + j, newArg);
-                    ++i;
-
-                }
-            }
-            --i; //decrement to compensate for the variable we replaced
-        }
-
-        if (argument == QLatin1String( "$RootNodeSwitch" )) {
-            //if the RootNodeSwitch argument has been added, we at least
-            //assume that the format of the switch has been added as well
-            Q_ASSERT(m_param.contains(RootNodeSwitch));
-
-            //we will decrement i afterwards
-            args.removeAt(i);
-
-            QString rootNode;
-            if (options.contains(QStringLiteral( "RootNode" ))) {
-                rootNode = options.value(QStringLiteral( "RootNode" )).toString();
-                qCDebug(ARK) << "Set root node " << rootNode;
-            }
-
-            if (!rootNode.isEmpty()) {
-                QStringList theSwitch = m_param.value(RootNodeSwitch).toStringList();
-                for (int j = 0; j < theSwitch.size(); ++j) {
-                    //get the argument part
-                    QString newArg = theSwitch.at(j);
-
-                    //substitute the $Path
-                    newArg.replace(QLatin1String( "$Path" ), rootNode);
-
-                    //put it in the arg list
-                    args.insert(i + j, newArg);
-                    ++i;
-
-                }
-            }
-            --i; //decrement to compensate for the variable we replaced
-        }
-
-        if (argument == QLatin1String( "$Files" )) {
-            args.removeAt(i);
-            for (int j = 0; j < files.count(); ++j) {
-                args.insert(i + j, escapeFileName(files.at(j).value<fileRootNodePair>().file));
-                ++i;
-            }
-            --i;
+    if (extractArgs.contains(QStringLiteral("$PasswordSwitch")) &&
+        options.value(QStringLiteral("PasswordProtectedHint")).toBool() &&
+        password().isEmpty()) {
+        qCDebug(ARK) << "Password hint enabled, querying user";
+        if (!passwordQuery()) {
+            return false;
         }
     }
 
-    bool useTmpExtractDir = options.value(QStringLiteral("DragAndDrop")).toBool();
-    QUrl destDir;
-    QTemporaryDir tmpExtractDir;
+    // Populate the argument list.
+    const QStringList args = substituteCopyVariables(extractArgs,
+                                                     files,
+                                                     options.value(QStringLiteral("PreservePaths")).toBool(),
+                                                     password(),
+                                                     options.value(QStringLiteral("RootNode"), QString()).toString());
+
+    QUrl destDir = QUrl(destinationDirectory);
+    QDir::setCurrent(destDir.adjusted(QUrl::RemoveScheme).url());
+
+    QString oldCurrentDir;
+    bool useTmpExtractDir = options.value(QStringLiteral("DragAndDrop")).toBool() ||
+                            options.value(QStringLiteral("AlwaysUseTmpDir")).toBool();
+    QTemporaryDir tmpExtractDir(QApplication::applicationName() + QLatin1Char('-'));
 
     if (useTmpExtractDir) {
         qCDebug(ARK) << "Using temporary extraction dir:" << tmpExtractDir.path();
@@ -281,19 +178,62 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
             failOperation();
             return false;
         }
+        oldCurrentDir = QDir::currentPath();
         destDir = QUrl(tmpExtractDir.path());
-    } else {
-        destDir = QUrl(destinationDirectory);
+        QDir::setCurrent(destDir.adjusted(QUrl::RemoveScheme).url());
     }
-    QDir::setCurrent(destDir.adjusted(QUrl::RemoveScheme).url());
 
     if (!runProcess(m_param.value(ExtractProgram).toStringList(), args)) {
         failOperation();
         return false;
     }
 
-    if (useTmpExtractDir) {
-        moveToFinalDest(files, destinationDirectory, tmpExtractDir);
+    if (options.value(QStringLiteral("AlwaysUseTmpDir")).toBool()) {
+        // unar exits with code 1 if extraction fails.
+        // This happens at least with wrong passwords or not enough space in the destination folder.
+        if (m_exitCode == 1) {
+            if (password().isEmpty()) {
+                qCWarning(ARK) << "Extraction aborted, destination folder might not have enough space.";
+                emit error(i18n("Extraction failed. Make sure that enough space is available."));
+            } else {
+                qCWarning(ARK) << "Extraction aborted, either the password is wrong or the destination folder doesn't have enough space.";
+                emit error(i18n("Extraction failed. Make sure you provided the correct password and that enough space is available."));
+                setPassword(QString());
+            }
+
+            emit finished(false);
+            failOperation();
+            // If we don't do this, the temporary directory will not autodelete itself upon destruction.
+            QDir::setCurrent(oldCurrentDir);
+
+            return false;
+        }
+
+        if (!options.value(QStringLiteral("DragAndDrop")).toBool()) {
+            if (!moveToDestination(QDir::current(), QDir(destinationDirectory), options[QStringLiteral("PreservePaths")].toBool())) {
+                emit error(i18ncp("@info",
+                                  "Could not move the extracted file to the destination directory.",
+                                  "Could not move the extracted files to the destination directory.",
+                                  files.size()));
+                emit finished(false);
+                return false;
+            }
+            // If we don't do this, the temporary directory will not autodelete itself upon destruction.
+            QDir::setCurrent(oldCurrentDir);
+        }
+    }
+
+    if (options.value(QStringLiteral("DragAndDrop")).toBool()) {
+        if (!moveDroppedFilesToDest(files, destinationDirectory)) {
+            emit error(i18ncp("@info",
+                              "Could not move the extracted file to the destination directory.",
+                              "Could not move the extracted files to the destination directory.",
+                              files.size()));
+            emit finished(false);
+            return false;
+        }
+        // If we don't do this, the temporary directory will not autodelete itself upon destruction.
+        QDir::setCurrent(oldCurrentDir);
     }
 
     emit finished(true);
@@ -460,7 +400,7 @@ bool CliInterface::runProcess(const QStringList& programNames, const QStringList
         return false;
     }
 
-    qCDebug(ARK) << "Executing" << programPath << arguments;
+    qCDebug(ARK) << "Executing" << programPath << arguments << "within directory" << QDir::currentPath();
 
     if (m_process) {
         m_process->waitForFinished();
@@ -500,6 +440,7 @@ bool CliInterface::runProcess(const QStringList& programNames, const QStringList
 
 void CliInterface::processFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
+    m_exitCode = exitCode;
     qCDebug(ARK) << "Process finished, exitcode:" << exitCode << "exitstatus:" << exitStatus;
 
     //if the m_process pointer is gone, then there is nothing to worry
@@ -528,7 +469,7 @@ void CliInterface::processFinished(int exitCode, QProcess::ExitStatus exitStatus
     }
 }
 
-bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &finalDest, const QTemporaryDir &tmpDir)
+bool CliInterface::moveDroppedFilesToDest(const QVariantList &files, const QString &finalDest)
 {
     // Move extracted files from a QTemporaryDir to the final destination.
 
@@ -541,7 +482,7 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
     foreach (const QVariant& file, files) {
 
         QFileInfo relEntry(file.value<fileRootNodePair>().file.remove(file.value<fileRootNodePair>().rootNode));
-        QFileInfo absSourceEntry(tmpDir.path() + QLatin1Char('/') + file.value<fileRootNodePair>().file);
+        QFileInfo absSourceEntry(QDir::current().absolutePath() + QLatin1Char('/') + file.value<fileRootNodePair>().file);
         QFileInfo absDestEntry(finalDestDir.path() + QLatin1Char('/') + relEntry.filePath());
 
         if (absSourceEntry.isDir()) {
@@ -597,7 +538,7 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
 
             // Create any parent directories.
             if (!finalDestDir.mkpath(relEntry.path())) {
-                qCWarning(ARK) << "Failed to create parent dirctory for file:" << absDestEntry.filePath();
+                qCWarning(ARK) << "Failed to create parent directory for file:" << absDestEntry.filePath();
             }
 
             // Move files to the final destination.
@@ -610,12 +551,222 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
     return true;
 }
 
+bool CliInterface::isEmptyDir(const QDir &dir)
+{
+    QDir d = dir;
+    d.setFilter(QDir::AllEntries | QDir::NoDotAndDotDot);
+
+    return d.count() == 0;
+}
+
+bool CliInterface::moveToDestination(const QDir &tempDir, const QDir &destDir, bool preservePaths)
+{
+    qCDebug(ARK) << "Moving extracted files from temp dir" << tempDir.path() << "to final destination" << destDir.path();
+
+    bool overwriteAll = false;
+    bool skipAll = false;
+
+    QDirIterator dirIt(tempDir.path(), QDir::AllEntries | QDir::NoDotAndDotDot, QDirIterator::Subdirectories);
+    while (dirIt.hasNext()) {
+        dirIt.next();
+
+        // We skip directories if:
+        // 1. We are not preserving paths
+        // 2. The dir is not empty. Only empty directories need to be explicitly moved.
+        // The non-empty ones are created by QDir::mkpath() below.
+        if (dirIt.fileInfo().isDir()) {
+            if (!preservePaths || !isEmptyDir(QDir(dirIt.filePath()))) {
+                continue;
+            }
+        }
+
+        QFileInfo relEntry;
+        if (preservePaths) {
+            relEntry = QFileInfo(dirIt.filePath().remove(tempDir.path() + QLatin1Char('/')));
+        } else {
+            relEntry = QFileInfo(dirIt.fileName());
+        }
+
+        QFileInfo absDestEntry(destDir.path() + QLatin1Char('/') + relEntry.filePath());
+
+        if (absDestEntry.exists()) {
+            qCWarning(ARK) << "File" << absDestEntry.absoluteFilePath() << "exists.";
+
+            Kerfuffle::OverwriteQuery query(absDestEntry.absoluteFilePath());
+            query.setNoRenameMode(true);
+            emit userQuery(&query);
+            query.waitForResponse();
+
+            if (query.responseOverwrite() || query.responseOverwriteAll()) {
+                if (query.responseOverwriteAll()) {
+                    overwriteAll = true;
+                }
+                if (!QFile::remove(absDestEntry.absoluteFilePath())) {
+                    qCWarning(ARK) << "Failed to remove" << absDestEntry.absoluteFilePath();
+                }
+
+            } else if (query.responseSkip() || query.responseAutoSkip()) {
+                if (query.responseAutoSkip()) {
+                    skipAll = true;
+                }
+                continue;
+            } else if (query.responseCancelled()) {
+                qCDebug(ARK) << "Copy action cancelled.";
+                return false;
+            }
+        } else if (skipAll) {
+            continue;
+        } else if (overwriteAll) {
+            if (!QFile::remove(absDestEntry.absoluteFilePath())) {
+                qCWarning(ARK) << "Failed to remove" << absDestEntry.absoluteFilePath();
+            }
+        }
+
+        if (preservePaths) {
+            // Create any parent directories.
+            if (!destDir.mkpath(relEntry.path())) {
+                qCWarning(ARK) << "Failed to create parent directory for file:" << absDestEntry.filePath();
+            }
+        }
+
+        // Move file to the final destination.
+        if (!QFile(dirIt.filePath()).rename(absDestEntry.absoluteFilePath())) {
+            qCWarning(ARK) << "Failed to move file" << dirIt.filePath() << "to final destination.";
+            return false;
+        }
+    }
+
+    return true;
+}
+
+QStringList CliInterface::substituteCopyVariables(const QStringList &extractArgs, const QVariantList &files, bool preservePaths, const QString &password, const QString &rootNode)
+{
+    // Required if we call this function from unit tests.
+    cacheParameterList();
+
+    QStringList args;
+    foreach (const QString& arg, extractArgs) {
+        qCDebug(ARK) << "Processing argument" << arg;
+
+        if (arg == QLatin1String("$Archive")) {
+            args << filename();
+            continue;
+        }
+
+        if (arg == QLatin1String("$PreservePathSwitch")) {
+            args << preservePathSwitch(preservePaths);
+            continue;
+        }
+
+        if (arg == QLatin1String("$PasswordSwitch")) {
+
+            args << passwordSwitch(password);
+            continue;
+        }
+
+        if (arg == QLatin1String("$RootNodeSwitch")) {
+            args << rootNodeSwitch(rootNode);
+            continue;
+        }
+
+        if (arg == QLatin1String("$Files")) {
+            args << copyFilesList(files);
+            continue;
+        }
+
+        // Simple argument (e.g. -kb in unrar), nothing to substitute, just add it to the list.
+        args << arg;
+    }
+
+    // Remove empty strings, if any.
+    args.removeAll(QString());
+
+    return args;
+}
+
+QString CliInterface::preservePathSwitch(bool preservePaths) const
+{
+    Q_ASSERT(m_param.contains(PreservePathSwitch));
+    const QStringList theSwitch = m_param.value(PreservePathSwitch).toStringList();
+    Q_ASSERT(theSwitch.size() == 2);
+
+    return (preservePaths ? theSwitch.at(0) : theSwitch.at(1));
+}
+
+QStringList CliInterface::passwordSwitch(const QString& password) const
+{
+    if (password.isEmpty()) {
+        return QStringList();
+    }
+
+    Q_ASSERT(m_param.contains(PasswordSwitch));
+
+    QStringList passwordSwitch = m_param.value(PasswordSwitch).toStringList();
+    Q_ASSERT(!passwordSwitch.isEmpty() && passwordSwitch.size() <= 2);
+
+    if (passwordSwitch.size() == 1) {
+        passwordSwitch[0].replace(QLatin1String("$Password"), password);
+    } else {
+        passwordSwitch[1] = password;
+    }
+
+    return passwordSwitch;
+}
+
+QStringList CliInterface::rootNodeSwitch(const QString &rootNode) const
+{
+    if (rootNode.isEmpty()) {
+        return QStringList();
+    }
+
+    Q_ASSERT(m_param.contains(RootNodeSwitch));
+
+    QStringList rootNodeSwitch = m_param.value(RootNodeSwitch).toStringList();
+    Q_ASSERT(!rootNodeSwitch.isEmpty() && rootNodeSwitch.size() <= 2);
+
+    if (rootNodeSwitch.size() == 1) {
+        rootNodeSwitch[0].replace(QLatin1String("$Path"), rootNode);
+    } else {
+        rootNodeSwitch[1] = rootNode;
+    }
+
+    return rootNodeSwitch;
+}
+
+QStringList CliInterface::copyFilesList(const QVariantList& files) const
+{
+    QStringList filesList;
+    foreach (const QVariant& f, files) {
+        filesList << escapeFileName(f.value<fileRootNodePair>().file);
+    }
+
+    return filesList;
+}
+
 void CliInterface::failOperation()
 {
     // TODO: Would be good to unit test #304764/#304178.
     doKill();
 }
 
+bool CliInterface::passwordQuery()
+{
+    Kerfuffle::PasswordNeededQuery query(filename());
+    emit userQuery(&query);
+    query.waitForResponse();
+
+    if (query.responseCancelled()) {
+        emit cancelled();
+        // There is no process running, so finished() must be emitted manually.
+        emit finished(false);
+        failOperation();
+        return false;
+    }
+
+    setPassword(query.password());
+    return true;
+}
+
 void CliInterface::readStdout(bool handleAll)
 {
     //when hacking this function, please remember the following:
@@ -660,6 +811,7 @@ void CliInterface::readStdout(bool handleAll)
 
     bool foundErrorMessage =
         (wrongPasswordMessage ||
+         checkForErrorMessage(QLatin1String(lines.last()), DiskFullPatterns) ||
          checkForErrorMessage(QLatin1String(lines.last()), ExtractionFailedPatterns) ||
          checkForPasswordPromptMessage(QLatin1String(lines.last())) ||
          checkForErrorMessage(QLatin1String(lines.last()), FileExistsExpression));
@@ -712,6 +864,7 @@ void CliInterface::handleLine(const QString& line)
     }
 
     if (m_operationMode == Copy) {
+
         if (checkForPasswordPromptMessage(line)) {
             qCDebug(ARK) << "Found a password prompt";
 
@@ -733,6 +886,13 @@ void CliInterface::handleLine(const QString& line)
             return;
         }
 
+        if (checkForErrorMessage(line, DiskFullPatterns)) {
+            qCWarning(ARK) << "Found disk full message:" << line;
+            emit error(i18nc("@info", "Extraction failed because the disk is full."));
+            failOperation();
+            return;
+        }
+
         if (checkForErrorMessage(line, WrongPasswordPatterns)) {
             qCWarning(ARK) << "Wrong password!";
             setPassword(QString());
@@ -742,7 +902,7 @@ void CliInterface::handleLine(const QString& line)
         }
 
         if (checkForErrorMessage(line, ExtractionFailedPatterns)) {
-            qCWarning(ARK) << "Error in extraction!!";
+            qCWarning(ARK) << "Error in extraction:" << line;
             emit error(i18n("Extraction failed because of an unexpected error."));
             failOperation();
             return;
@@ -948,7 +1108,7 @@ void CliInterface::substituteListVariables(QStringList& params)
             Q_ASSERT(m_param.contains(PasswordSwitch));
 
             //we will set it afterwards, if there is a password
-            params[i] = QString();
+            params.removeAt(i);
 
             QString pass = password();
 
@@ -963,9 +1123,11 @@ void CliInterface::substituteListVariables(QStringList& params)
                     newArg.replace(QLatin1String("$Password"), pass);
 
                     //put it in the arg list
-                    params[i] = newArg;
+                    params.insert(i + j, newArg);
+                    ++i;
                 }
             }
+            --i;
         }
     }
 }
