@@ -35,6 +35,7 @@
 #include "kerfuffle/jobs.h"
 #include "kerfuffle/settings.h"
 #include "kerfuffle/previewsettingspage.h"
+#include "kerfuffle/propertiesdialog.h"
 
 #include <KAboutData>
 #include <KActionCollection>
@@ -59,6 +60,7 @@
 #include <QMimeData>
 #include <QMouseEvent>
 #include <QScopedPointer>
+#include <QStatusBar>
 #include <QPointer>
 #include <QSplitter>
 #include <QTimer>
@@ -174,7 +176,8 @@ Part::~Part()
     ArkSettings::setShowInfoPanel(m_showInfoPanelAction->isChecked());
     ArkSettings::self()->save();
 
-    m_extractFilesAction->menu()->deleteLater();
+    m_extractArchiveAction->menu()->deleteLater();
+    m_extractAction->menu()->deleteLater();
 }
 
 KAboutData *Part::createAboutData()
@@ -206,10 +209,11 @@ void Part::extractSelectedFilesTo(const QString& localPath)
         return;
     }
 
-    const QUrl url(localPath);
+    const QUrl url = QUrl::fromUserInput(localPath, QString());
     KIO::StatJob* statJob = nullptr;
 
-    if (!url.scheme().isEmpty()) {
+    // Try to resolve the URL to a local path.
+    if (!url.isLocalFile() && !url.scheme().isEmpty()) {
         statJob = KIO::mostLocalUrl(url);
 
         if (!statJob->exec() || statJob->error() != 0) {
@@ -220,6 +224,13 @@ void Part::extractSelectedFilesTo(const QString& localPath)
     const QString destination = statJob ? statJob->statResult().stringValue(KIO::UDSEntry::UDS_LOCAL_PATH) : localPath;
     delete statJob;
 
+    // The URL could not be resolved to a local path.
+    if (!url.isLocalFile() && destination.isEmpty()) {
+        qCWarning(ARK) << "Ark cannot extract to non-local destination:" << localPath;
+        KMessageBox::sorry(widget(), xi18nc("@info", "Ark can only extract to local destinations."));
+        return;
+    }
+
     qCDebug(ARK) << "Extract to" << destination;
 
     Kerfuffle::ExtractionOptions options;
@@ -279,49 +290,57 @@ void Part::setupActions()
     connect(m_showInfoPanelAction, &QAction::triggered,
             this, &Part::slotToggleInfoPanel);
 
-    m_saveAsAction = KStandardAction::saveAs(this, SLOT(slotSaveAs()), actionCollection());
+    m_saveAsAction = actionCollection()->addAction(KStandardAction::SaveAs, QStringLiteral("ark_file_save_as"), this, SLOT(slotSaveAs()));
 
     m_openFileAction = actionCollection()->addAction(QStringLiteral("openfile"));
-    m_openFileAction->setText(i18nc("open a file with external program", "&Open File"));
+    m_openFileAction->setText(i18nc("open a file with external program", "&Open"));
     m_openFileAction->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
-    m_openFileAction->setStatusTip(i18n("Click to open the selected file with the associated application"));
+    m_openFileAction->setToolTip(i18nc("@info:tooltip", "Click to open the selected file with the associated application"));
     connect(m_openFileAction, SIGNAL(triggered(bool)), m_signalMapper, SLOT(map()));
     m_signalMapper->setMapping(m_openFileAction, OpenFile);
 
     m_openFileWithAction = actionCollection()->addAction(QStringLiteral("openfilewith"));
-    m_openFileWithAction->setText(i18nc("open a file with external program", "Open File &With..."));
+    m_openFileWithAction->setText(i18nc("open a file with external program", "Open &With..."));
     m_openFileWithAction->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
-    m_openFileWithAction->setStatusTip(i18n("Click to open the selected file with an external program"));
+    m_openFileWithAction->setToolTip(i18nc("@info:tooltip", "Click to open the selected file with an external program"));
     connect(m_openFileWithAction, SIGNAL(triggered(bool)), m_signalMapper, SLOT(map()));
     m_signalMapper->setMapping(m_openFileWithAction, OpenFileWith);
 
     m_previewAction = actionCollection()->addAction(QStringLiteral("preview"));
     m_previewAction->setText(i18nc("to preview a file inside an archive", "Pre&view"));
     m_previewAction->setIcon(QIcon::fromTheme(QStringLiteral("document-preview-archive")));
-    m_previewAction->setStatusTip(i18n("Click to preview the selected file"));
+    m_previewAction->setToolTip(i18nc("@info:tooltip", "Click to preview the selected file"));
     actionCollection()->setDefaultShortcut(m_previewAction, Qt::CTRL + Qt::Key_P);
     connect(m_previewAction, SIGNAL(triggered(bool)), m_signalMapper, SLOT(map()));
     m_signalMapper->setMapping(m_previewAction, Preview);
 
-    m_extractFilesAction = actionCollection()->addAction(QStringLiteral("extract"));
-    m_extractFilesAction->setText(i18n("E&xtract"));
-    m_extractFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-extract")));
-    m_extractFilesAction->setStatusTip(i18n("Click to open an extraction dialog, where you can choose to extract either all files or just the selected ones"));
-    actionCollection()->setDefaultShortcut(m_extractFilesAction, Qt::CTRL + Qt::Key_E);
-    connect(m_extractFilesAction, &QAction::triggered,
-            this, &Part::slotExtractFiles);
+    m_extractArchiveAction = actionCollection()->addAction(QStringLiteral("extract_all"));
+    m_extractArchiveAction->setText(i18nc("@action:inmenu", "E&xtract All"));
+    m_extractArchiveAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-extract")));
+    m_extractArchiveAction->setToolTip(i18n("Click to open an extraction dialog, where you can choose how to extract all the files in the archive"));
+    actionCollection()->setDefaultShortcut(m_extractArchiveAction, Qt::CTRL + Qt::SHIFT + Qt::Key_E);
+    connect(m_extractArchiveAction, &QAction::triggered,
+            this, &Part::slotExtractArchive);
+
+    m_extractAction = actionCollection()->addAction(QStringLiteral("extract"));
+    m_extractAction->setText(i18nc("@action:inmenu", "&Extract"));
+    m_extractAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-extract")));
+    actionCollection()->setDefaultShortcut(m_extractAction, Qt::CTRL + Qt::Key_E);
+    m_extractAction->setToolTip(i18n("Click to open an extraction dialog, where you can choose to extract either all files or just the selected ones"));
+    connect(m_extractAction, &QAction::triggered,
+            this, &Part::slotShowExtractionDialog);
 
     m_addFilesAction = actionCollection()->addAction(QStringLiteral("add"));
     m_addFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-insert")));
     m_addFilesAction->setText(i18n("Add &File..."));
-    m_addFilesAction->setStatusTip(i18n("Click to add files to the archive"));
+    m_addFilesAction->setToolTip(i18nc("@info:tooltip", "Click to add files to the archive"));
     connect(m_addFilesAction, SIGNAL(triggered(bool)),
             this, SLOT(slotAddFiles()));
 
     m_addDirAction = actionCollection()->addAction(QStringLiteral("add-dir"));
     m_addDirAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-insert-directory")));
     m_addDirAction->setText(i18n("Add Fo&lder..."));
-    m_addDirAction->setStatusTip(i18n("Click to add a folder to the archive"));
+    m_addDirAction->setToolTip(i18nc("@info:tooltip", "Click to add a folder to the archive"));
     connect(m_addDirAction, &QAction::triggered,
             this, &Part::slotAddDir);
 
@@ -329,13 +348,23 @@ void Part::setupActions()
     m_deleteFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-remove")));
     m_deleteFilesAction->setText(i18n("De&lete"));
     actionCollection()->setDefaultShortcut(m_deleteFilesAction, Qt::Key_Delete);
-    m_deleteFilesAction->setStatusTip(i18n("Click to delete the selected files"));
+    m_deleteFilesAction->setToolTip(i18nc("@info:tooltip", "Click to delete the selected files"));
     connect(m_deleteFilesAction, &QAction::triggered,
             this, &Part::slotDeleteFiles);
 
+    m_propertiesAction = actionCollection()->addAction(QStringLiteral("properties"));
+    m_propertiesAction->setIcon(QIcon::fromTheme(QStringLiteral("document-properties")));
+    m_propertiesAction->setText(i18nc("@action:inmenu", "&Properties"));
+    actionCollection()->setDefaultShortcut(m_propertiesAction, Qt::ALT + Qt::Key_Return);
+    m_propertiesAction->setToolTip(i18nc("@info:tooltip", "Click to see properties for archive"));
+    connect(m_propertiesAction, &QAction::triggered,
+            this, &Part::slotShowProperties);
+
     connect(m_signalMapper, SIGNAL(mapped(int)), this, SLOT(slotOpenEntry(int)));
 
     updateActions();
+    updateQuickExtractMenu(m_extractArchiveAction);
+    updateQuickExtractMenu(m_extractAction);
 }
 
 void Part::updateActions()
@@ -354,8 +383,10 @@ void Part::updateActions()
                                 isPreviewable &&
                                 !isDirectory &&
                                 (selectedEntriesCount == 1));
-    m_extractFilesAction->setEnabled(!isBusy() &&
-                                     (m_model->rowCount() > 0));
+    m_extractArchiveAction->setEnabled(!isBusy() &&
+                                       (m_model->rowCount() > 0));
+    m_extractAction->setEnabled(!isBusy() &&
+                                (m_model->rowCount() > 0));
     m_saveAsAction->setEnabled(!isBusy() &&
                                m_model->rowCount() > 0);
     m_addFilesAction->setEnabled(!isBusy() &&
@@ -373,21 +404,38 @@ void Part::updateActions()
                                      isPreviewable &&
                                      !isDirectory &&
                                      (selectedEntriesCount == 1));
+    m_propertiesAction->setEnabled(!isBusy() &&
+                                   m_model->archive());
+}
+
+void Part::updateQuickExtractMenu(QAction *extractAction)
+{
+    if (!extractAction) {
+        return;
+    }
+
+    QMenu *menu = extractAction->menu();
 
-    QMenu *menu = m_extractFilesAction->menu();
     if (!menu) {
-        menu = new QMenu;
-        m_extractFilesAction->setMenu(menu);
+        menu = new QMenu();
+        extractAction->setMenu(menu);
         connect(menu, &QMenu::triggered,
                 this, &Part::slotQuickExtractFiles);
 
         // Remember to keep this action's properties as similar to
-        // m_extractFilesAction's as possible (except where it does not make
+        // extractAction's as possible (except where it does not make
         // sense, such as the text or the shortcut).
         QAction *extractTo = menu->addAction(i18n("Extract To..."));
-        extractTo->setIcon(m_extractFilesAction->icon());
-        extractTo->setStatusTip(m_extractFilesAction->statusTip());
-        connect(extractTo, &QAction::triggered, this, &Part::slotExtractFiles);
+        extractTo->setIcon(extractAction->icon());
+        extractTo->setToolTip(extractAction->toolTip());
+
+        if (extractAction == m_extractArchiveAction) {
+            connect(extractTo, &QAction::triggered,
+                    this, &Part::slotExtractArchive);
+        } else {
+            connect(extractTo, &QAction::triggered,
+                    this, &Part::slotShowExtractionDialog);
+        }
 
         menu->addSeparator();
 
@@ -405,8 +453,10 @@ void Part::updateActions()
 
     for (int i = 0; i < qMin(10, dirHistory.size()); ++i) {
         const QString dir = QUrl(dirHistory.value(i)).toString(QUrl::RemoveScheme | QUrl::NormalizePathSegments | QUrl::PreferLocalFile);
-        QAction *newAction = menu->addAction(dir);
-        newAction->setData(dir);
+        if (QDir(dir).exists()) {
+            QAction *newAction = menu->addAction(dir);
+            newAction->setData(dir);
+        }
     }
 }
 
@@ -453,62 +503,31 @@ bool Part::openFile()
 {
     qCDebug(ARK) << "Attempting to open archive" << localFilePath();
 
-    const QString localFile(localFilePath());
-    const QFileInfo localFileInfo(localFile);
-    const bool creatingNewArchive =
-        arguments().metaData()[QStringLiteral("createNewArchive")] == QLatin1String("true");
-
-    if (localFileInfo.isDir()) {
-        KMessageBox::error(widget(), xi18nc("@info",
-                                            "<filename>%1</filename> is a directory.",
-                                            localFile));
+    if (!isLocalFileValid()) {
         return false;
     }
 
-    if (creatingNewArchive) {
-        if (localFileInfo.exists()) {
-            int overwrite =  KMessageBox::questionYesNo(widget(), xi18nc("@info", "The archive <filename>%1</filename> already exists. Would you like to open it instead?", localFile), i18nc("@title:window", "File Exists"), KGuiItem(i18n("Open File")), KStandardGuiItem::cancel());
-
-            if (overwrite == KMessageBox::No) {
-                return false;
-            }
-        }
-    } else {
-        if (!localFileInfo.exists()) {
-            KMessageBox::sorry(widget(), xi18nc("@info", "The archive <filename>%1</filename> was not found.", localFile), i18nc("@title:window", "Error Opening Archive"));
-            return false;
-        }
-
-        if (!localFileInfo.isReadable()) {
-            displayMsgWidget(KMessageWidget::Warning, xi18nc("@info", "The archive <filename>%1</filename> could not be loaded, as it was not possible to read from it.", localFile));
-            return false;
-        }
-    }
-
-    QScopedPointer<Kerfuffle::Archive> archive(Kerfuffle::Archive::create(localFile, m_model));
-
+    QScopedPointer<Kerfuffle::Archive> archive(Kerfuffle::Archive::create(localFilePath(), m_model));
     Q_ASSERT(archive);
 
     if (archive->error() == NoPlugin) {
-        KMessageBox::sorry(widget(),
-                           xi18nc("@info", "Ark was not able to open <filename>%1</filename>. No suitable plugin found."
-                                  "<nl/><nl/>Ark does not seem to support this file type.",
-                                  localFile.section(QLatin1Char('/'), -1)),
-                           i18nc("@title:window", "Error Opening Archive"));
+        displayMsgWidget(KMessageWidget::Error, xi18nc("@info", "Ark was not able to open <filename>%1</filename>. No suitable plugin found.<nl/>"
+                                                                "Ark does not seem to support this file type.",
+                                                                QFileInfo(localFilePath()).fileName()));
         return false;
-    } else if (archive->error() == FailedPlugin) {
-        KMessageBox::sorry(widget(),
-                           xi18nc("@info", "Ark was not able to open <filename>%1</filename>. Failed to load a suitable plugin."
-                                  "<nl/><nl/>Make sure any executables needed to handle the archive type are installed.",
-                                  localFile.section(QLatin1Char('/'), -1)),
-                           i18nc("@title:window", "Error Opening Archive"));
+    }
+
+    if (archive->error() == FailedPlugin) {
+        displayMsgWidget(KMessageWidget::Error, xi18nc("@info", "Ark was not able to open <filename>%1</filename>. Failed to load a suitable plugin.<nl/>"
+                                                                "Make sure any executables needed to handle the archive type are installed.",
+                                                                QFileInfo(localFilePath()).fileName()));
         return false;
     }
 
     Q_ASSERT(archive->isValid());
 
     // Plugin loaded successfully.
-    KJob *job = m_model->setArchive(archive.take(), localFileInfo.exists());
+    KJob *job = m_model->setArchive(archive.take());
     if (job) {
         registerJob(job);
         job->start();
@@ -519,16 +538,13 @@ bool Part::openFile()
     m_infoPanel->setIndex(QModelIndex());
 
     if (arguments().metaData()[QStringLiteral("showExtractDialog")] == QLatin1String("true")) {
-        QTimer::singleShot(0, this, &Part::slotExtractFiles);
+        QTimer::singleShot(0, this, &Part::slotShowExtractionDialog);
     }
 
     const QString password = arguments().metaData()[QStringLiteral("encryptionPassword")];
     if (!password.isEmpty()) {
-        m_model->setPassword(password);
-
-        if (arguments().metaData()[QStringLiteral("encryptHeader")] == QLatin1String("true")) {
-            m_model->enableHeaderEncryption();
-        }
+        m_model->encryptArchive(password,
+                                arguments().metaData()[QStringLiteral("encryptHeader")] == QLatin1String("true"));
     }
 
     return true;
@@ -552,12 +568,71 @@ KConfigSkeleton *Part::config() const
 QList<Kerfuffle::SettingsPage*> Part::settingsPages(QWidget *parent) const
 {
     QList<SettingsPage*> pages;
-    pages.append(new ExtractionSettingsPage(parent, i18n("Extraction settings"), QStringLiteral("archive-extract")));
-    pages.append(new PreviewSettingsPage(parent, i18n("Preview settings"), QStringLiteral("document-preview-archive")));
+    pages.append(new ExtractionSettingsPage(parent, i18nc("@title:tab", "Extraction Settings"), QStringLiteral("archive-extract")));
+    pages.append(new PreviewSettingsPage(parent, i18nc("@title:tab", "Preview Settings"), QStringLiteral("document-preview-archive")));
 
     return pages;
 }
 
+bool Part::isLocalFileValid()
+{
+    const QString localFile = localFilePath();
+    const QFileInfo localFileInfo(localFile);
+    const bool creatingNewArchive = arguments().metaData()[QStringLiteral("createNewArchive")] == QLatin1String("true");
+
+    if (localFileInfo.isDir()) {
+        displayMsgWidget(KMessageWidget::Error, xi18nc("@info",
+                                                       "<filename>%1</filename> is a directory.",
+                                                       localFile));
+        return false;
+    }
+
+    if (creatingNewArchive) {
+        if (localFileInfo.exists()) {
+            if (!confirmAndDelete(localFile)) {
+                displayMsgWidget(KMessageWidget::Error, xi18nc("@info",
+                                                               "Could not overwrite <filename>%1</filename>. Check whether you have write permission.",
+                                                               localFile));
+                return false;
+            }
+        }
+
+        displayMsgWidget(KMessageWidget::Information, xi18nc("@info", "The archive <filename>%1</filename> will be created as soon as you add a file.", localFile));
+    } else {
+        if (!localFileInfo.exists()) {
+            displayMsgWidget(KMessageWidget::Error, xi18nc("@info", "The archive <filename>%1</filename> was not found.", localFile));
+            return false;
+        }
+
+        if (!localFileInfo.isReadable()) {
+            displayMsgWidget(KMessageWidget::Error, xi18nc("@info", "The archive <filename>%1</filename> could not be loaded, as it was not possible to read from it.", localFile));
+            return false;
+        }
+    }
+
+    return true;
+}
+
+bool Part::confirmAndDelete(const QString &targetFile)
+{
+    QFileInfo targetInfo(targetFile);
+    const auto buttonCode = KMessageBox::warningYesNo(widget(),
+                                                      xi18nc("@info",
+                                                             "The archive <filename>%1</filename> already exists. Do you wish to overwrite it?",
+                                                             targetInfo.fileName()),
+                                                      i18nc("@title:window", "File Exists"),
+                                                      KGuiItem(i18nc("@action:button", "Overwrite")),
+                                                      KStandardGuiItem::cancel());
+
+    if (buttonCode != KMessageBox::Yes || !targetInfo.isWritable()) {
+        return false;
+    }
+
+    qCDebug(ARK) << "Removing file" << targetFile;
+
+    return QFile(targetFile).remove();
+}
+
 void Part::slotLoadingStarted()
 {
 }
@@ -567,11 +642,9 @@ void Part::slotLoadingFinished(KJob *job)
     if (job->error()) {
         if (arguments().metaData()[QStringLiteral("createNewArchive")] != QLatin1String("true")) {
             if (job->error() != KJob::KilledJobError) {
-                KMessageBox::error(widget(),
-                                   xi18nc("@info", "Loading the archive <filename>%1</filename> failed with the following error:<nl/><nl/><message>%2</message>",
-                                          localFilePath(),
-                                          job->errorText()),
-                                   i18nc("@title:window", "Error Opening Archive"));
+                displayMsgWidget(KMessageWidget::Error, xi18nc("@info", "Loading the archive <filename>%1</filename> failed with the following error:<nl/><message>%2</message>",
+                                                               localFilePath(),
+                                                               job->errorText()));
             }
 
             // The file failed to open, so reset the open archive, info panel and caption.
@@ -597,7 +670,11 @@ void Part::slotLoadingFinished(KJob *job)
 
     updateActions();
 
-    if (m_model->archive() && !m_model->archive()->comment().isEmpty()) {
+    if (!m_model->archive()) {
+        return;
+    }
+
+    if (!m_model->archive()->comment().isEmpty()) {
         m_commentView->setPlainText(m_model->archive()->comment());
         m_commentBox->show();
         m_commentSplitter->setSizes(QList<int>() << m_view->height() * 0.6 << 1);
@@ -605,12 +682,28 @@ void Part::slotLoadingFinished(KJob *job)
         m_commentView->clear();
         m_commentBox->hide();
     }
+
+    if (m_model->rowCount() == 0) {
+        qCWarning(ARK) << "No entry listed by the plugin";
+        displayMsgWidget(KMessageWidget::Warning, xi18nc("@info", "The archive is empty or Ark could not open its content."));
+    } else if (m_model->rowCount() == 1) {
+        if (m_model->archive()->mimeType().inherits(QStringLiteral("application/x-cd-image")) &&
+            m_model->entryForIndex(m_model->index(0, 0))[FileName].toString() == QLatin1String("README.TXT")) {
+            qCWarning(ARK) << "Detected ISO image with UDF filesystem";
+            displayMsgWidget(KMessageWidget::Warning, xi18nc("@info", "Ark does not currently support ISO files with UDF filesystem."));
+        }
+    }
 }
 
 void Part::setReadyGui()
 {
     QApplication::restoreOverrideCursor();
     m_busy = false;
+
+    if (m_statusBarExtension->statusBar()) {
+        m_statusBarExtension->statusBar()->hide();
+    }
+
     m_view->setEnabled(true);
     updateActions();
 }
@@ -619,6 +712,11 @@ void Part::setBusyGui()
 {
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
     m_busy = true;
+
+    if (m_statusBarExtension->statusBar()) {
+        m_statusBarExtension->statusBar()->show();
+    }
+
     m_view->setEnabled(false);
     updateActions();
 }
@@ -761,7 +859,7 @@ void Part::slotWatchedFileModified(const QString& file)
     if (KMessageBox::questionYesNo(widget(),
                                xi18n("The file <filename>%1</filename> was modified. Do you want to update the archive?",
                                      prettyFilename),
-                               i18n("File modified")) == KMessageBox::Yes) {
+                               i18nc("@title:window", "File Modified")) == KMessageBox::Yes) {
         QStringList list = QStringList() << file;
 
         qCDebug(ARK) << "Updating file" << file << "with path" << relPath;
@@ -795,7 +893,16 @@ QString Part::detectSubfolder() const
     return m_model->archive()->subfolderName();
 }
 
-void Part::slotExtractFiles()
+void Part::slotExtractArchive()
+{
+    if (m_view->selectionModel()->selectedRows().count() > 0) {
+        m_view->selectionModel()->clear();
+    }
+
+    slotShowExtractionDialog();
+}
+
+void Part::slotShowExtractionDialog()
 {
     if (!m_model) {
         return;
@@ -818,8 +925,9 @@ void Part::slotExtractFiles()
     dialog.data()->restoreWindowSize();
 
     if (dialog.data()->exec()) {
-        //this is done to update the quick extract menu
-        updateActions();
+
+        updateQuickExtractMenu(m_extractArchiveAction);
+        updateQuickExtractMenu(m_extractAction);
 
         QVariantList files;
 
@@ -1050,12 +1158,16 @@ void Part::slotDeleteFilesDone(KJob* job)
 
 void Part::slotDeleteFiles()
 {
-    const int reallyDelete =
+    const int selectionsCount = m_view->selectionModel()->selectedRows().count();
+    const auto reallyDelete =
         KMessageBox::questionYesNo(widget(),
-                                   i18n("Deleting these files is not undoable. Are you sure you want to do this?"),
-                                   i18nc("@title:window", "Delete files"),
+                                   i18ncp("@info",
+                                          "Deleting this file is not undoable. Are you sure you want to do this?",
+                                          "Deleting these files is not undoable. Are you sure you want to do this?",
+                                          selectionsCount),
+                                   i18ncp("@title:window", "Delete File", "Delete Files", selectionsCount),
                                    KStandardGuiItem::del(),
-                                   KStandardGuiItem::cancel(),
+                                   KStandardGuiItem::no(),
                                    QString(),
                                    KMessageBox::Dangerous | KMessageBox::Notify);
 
@@ -1070,6 +1182,13 @@ void Part::slotDeleteFiles()
     job->start();
 }
 
+void Part::slotShowProperties()
+{
+    QPointer<Kerfuffle::PropertiesDialog> dialog(new Kerfuffle::PropertiesDialog(0,
+                                                                                 m_model->archive()));
+    dialog.data()->show();
+}
+
 void Part::slotToggleInfoPanel(bool visible)
 {
     if (visible) {
